@@ -1,20 +1,19 @@
+import io
 import json
 import logging
-import pathlib
+import os
+import tarfile
 import tempfile
 import time
-from typing import Dict, List, Union
+from typing import Dict, List
 
-import h5py
-import numpy as np
-import pandas as pd
 import requests
 import yaml
 
-from baseten.baseten_artifact import BasetenArtifact
 from baseten.common import api
 from baseten.common.core import raises_api_error
-from baseten.common.util import coerce_data_as_numpy_array
+from baseten.common.lib_support import (build_h5_data_object,
+                                        coerce_input_to_json)
 
 logger = logging.getLogger(__name__)
 
@@ -22,26 +21,6 @@ REQUIREMENTS_INSTALLATION_STATUS_RETRY_INTERVAL_SEC = 3
 REQUIREMENTS_INSTALLATION_STATUS_MAX_TRIES = 20
 
 
-def _build_h5_data_object(feature_data_np, target_data, metadata, data_temp_directory):
-    data_temp_file = pathlib.PurePath(data_temp_directory, 'tmp.h5')
-    data = {'features': feature_data_np}
-    if np.any(target_data):
-        data['targets'] = coerce_data_as_numpy_array(target_data)
-    if np.any(metadata):
-        data['metadata'] = json.dumps(metadata)
-
-    h5_data = h5py.File(data_temp_file, mode='w')
-    h5_sample_data_group = h5_data.create_group('sample_data')
-    for key, np_obj in data.items():
-        if key == 'metadata':
-            dt = h5py.string_dtype(encoding='utf-8')
-            h5_sample_data_group.create_dataset(key, data=np_obj, dtype=dt)
-        else:
-            h5_sample_data_group.create_dataset(key, data=np_obj)
-    h5_data.close()
-    return data_temp_file
-
-
 @raises_api_error
 def install_requirements(req_filepath: str):
     with open(req_filepath, 'r') as fp:
@@ -80,7 +59,8 @@ class BasetenDeployedModel:
     """A model backed by baseten serving. Provide either the model_id or the model_version_id."""
     def __init__(self, model_id: str = None,
                  model_version_id: str = None,
-                 model_name: str = None):
+                 model_name: str = None,
+                 model_zoo_name: str = None):
         if not model_id and not model_version_id:
             raise ValueError('Either model_id or model_version_id must be provided.')
 
@@ -91,6 +71,7 @@ class BasetenDeployedModel:
         self._model_version_id = model_version_id
         self._model_name = model_name
         self._model_config = None
+        self.model_zoo_name = model_zoo_name
 
     @property
     def model_version_id(self):
@@ -98,8 +79,8 @@ class BasetenDeployedModel:
 
     @raises_api_error
     def predict(self,
-                inputs: Union[List, pd.DataFrame, np.ndarray],
-                metadata: Union[pd.DataFrame, List[Dict]] = None) -> List[List]:
+                inputs,
+                metadata=None) -> List[List]:
         """Invokes the model given the input dataframe.
 
         Args:
@@ -115,23 +96,15 @@ class BasetenDeployedModel:
             TypeError: If the provided inputs is not of a supported type.
             ApiError: If there was an error communicating with the server.
         """
-        if isinstance(inputs, pd.DataFrame):
-            inputs_list = inputs.to_dict('records')
-        elif isinstance(inputs, (list, np.ndarray)):
-            inputs_np_array = np.array(inputs)
-            inputs_list = inputs_np_array.tolist()
-        else:
-            raise TypeError('predict can be called with either a list, a pandas DataFrame, or a numpy array.')
 
-        if isinstance(metadata, pd.DataFrame):
-            metadata = metadata.to_dict(orient='records')
+        inputs_list, metadata = coerce_input_to_json(inputs, metadata)
 
         if self._model_version_id:
             return api.predict_for_model_version(self._model_version_id, inputs_list, metadata)
         return api.predict_for_model(self._model_id, inputs_list, metadata)
 
     @raises_api_error
-    def update_model_config(self, model_config_file_path: str):
+    def update_model_features(self, model_config_file_path: str):
         """Update the model's feature names and output class labels (if any) based on the config
         found at `model_config_file_path`
 
@@ -141,7 +114,7 @@ class BasetenDeployedModel:
         config_yaml = yaml.safe_load(open(model_config_file_path, 'r'))
         feature_names = list(config_yaml['model_features']['features'])
         class_labels = config_yaml.get('model_class_labels', [])
-        api.update_model_config(self._model_version_id, feature_names, class_labels)
+        api.update_model_features(self._model_version_id, feature_names, class_labels)
 
     @raises_api_error
     def set_primary(self):
@@ -155,8 +128,8 @@ class BasetenDeployedModel:
 
     @raises_api_error
     def upload_sample_data(self,
-                           feature_data: Union[np.ndarray, pd.DataFrame, List[List]],
-                           target_data: Union[np.ndarray, pd.DataFrame, List] = None,
+                           feature_data,
+                           target_data=None,
                            metadata: List[Dict] = None,
                            data_name: str = 'validation_data',
                            ) -> Dict:
@@ -177,14 +150,14 @@ class BasetenDeployedModel:
         Returns:
             Dict: The status of the upload.
         """
+        if not self._model_version_id:
+            raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id.')
 
-        feature_data_np = coerce_data_as_numpy_array(feature_data)
-
-        signed_s3_upload_post = api.signed_s3_upload_post(self.model_version_id, data_name)
+        signed_s3_upload_post = api.signed_s3_upload_post(data_name)
         logger.debug(f'Signed s3 upload post:\n{json.dumps(signed_s3_upload_post, indent=4)}')
 
         with tempfile.TemporaryDirectory() as data_temp_directory:
-            data_temp_file = _build_h5_data_object(feature_data_np, target_data, metadata, data_temp_directory)
+            data_temp_file = build_h5_data_object(feature_data, target_data, metadata, data_temp_directory)
             files = {'file': (f'{data_name}.h5', open(data_temp_file, 'rb'))}
             form_fields = signed_s3_upload_post['form_fields']
             form_fields['AWSAccessKeyId'] = form_fields.pop('aws_access_key_id')
@@ -192,13 +165,14 @@ class BasetenDeployedModel:
             requests.post(signed_s3_upload_post['url'], data=form_fields, files=files)
         return api.register_data_for_model(s3_key, self.model_version_id, data_name)
 
+    @raises_api_error
     def artifacts(self):
+        from baseten.baseten_artifact import BasetenArtifact
         if self._model_version_id:
             return [
                 BasetenArtifact(**instance) for instance in api.model_version_linked_artifacts(self._model_version_id)
             ]
-        elif self._model_id:
-            return [BasetenArtifact(**instance) for instance in api.model_linked_artifacts(self._model_id)]
+        return [BasetenArtifact(**instance) for instance in api.model_linked_artifacts(self._model_id)]
 
     def __repr__(self):
         attr_info = []
@@ -210,3 +184,91 @@ class BasetenDeployedModel:
             attr_info.append(f'name={self._model_name}')
         info_str = '\n  '.join(attr_info)
         return f'BasetenDeployedModel<\n  {info_str}\n>'
+
+    @raises_api_error
+    def log_metrics(self, metrics: dict) -> None:
+        if not metrics:
+            raise ValueError('metrics must be provided.')
+        if not self._model_version_id and not self._model_id:
+            raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.')
+
+        if self._model_id:
+            api.create_model_metrics(metric_data=metrics, model_id=self._model_id)
+        elif self._model_version_id:
+            api.create_model_metrics(metric_data=metrics, model_version_id=self._model_version_id)
+        logger.info('✅ Metrics logged successfully ✅')
+
+    @raises_api_error
+    def get_metrics(self) -> dict:
+        if self._model_id:
+            return api.model_metrics(self._model_id)
+        elif self._model_version_id:
+            return api.model_version_metrics(self._model_version_id)
+
+        raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.')
+
+    @raises_api_error
+    def log_config(self, config: dict) -> None:
+        if not config:
+            raise ValueError('config must be provided.')
+        if not self._model_version_id and not self._model_id:
+            raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.')
+
+        if self._model_id:
+            api.create_model_configuration(configuration=config, model_id=self._model_id)
+        elif self._model_version_id:
+            api.create_model_configuration(configuration=config, model_version_id=self._model_version_id)
+        logger.info('✅ Config logged successfully ✅')
+
+    @raises_api_error
+    def get_config(self) -> dict:
+        if self._model_id:
+            return api.model_configuration(self._model_id)
+        elif self._model_version_id:
+            return api.model_version_configuration(self._model_version_id)
+
+        raise ValueError('Please use on a BasetenDeployedModel instantiated with a model_version_id or model_id.')
+
+    @raises_api_error
+    def add_artifact(
+        self,
+        name: str,
+        list_of_files: List[str],
+        description: str = None,
+    ):
+        """Create an artifact in the BaseTen backend to track
+
+            Args:
+                name (str): The name of the artifact to be created.
+                list_of_files (List[str]): A list of files to be uploaded.
+                description (str, optional): An optional description of the artifact
+                model_id (str, optional): A model_id to associate with the artifact
+                model_version_id (str, optional): A model_version_id to associate with the artifact
+            Returns:
+                BasetenArtifact
+
+            Raises:
+                ApiError: If there was an error communicating with the server.
+            """
+        from baseten.baseten_artifact import create_artifact
+        if self._model_version_id:
+            artifact = create_artifact(
+                name, list_of_files, description=description, model_version_id=self._model_version_id
+            )
+        else:
+            artifact = create_artifact(
+                name, list_of_files, description=description, model_id=self._model_id
+            )
+        return artifact
+
+    @raises_api_error
+    def pull(self, directory="."):
+        response = requests.get(api.get_model_s3_download_url(self.model_version_id))
+        with tarfile.open(name=None, fileobj=io.BytesIO(response.content)) as t:
+            t.extractall(os.path.join(directory, self.model_version_id))
+        return response
+
+
+@raises_api_error
+def models_summary():
+    return api.models_summary()
