@@ -1,15 +1,15 @@
 """Console script for baseten."""
 import configparser
 import functools
-import json
 import logging
 import sys
-from urllib.parse import urlparse
 
 import click
+from truss.cli import cli_group as truss_commands
 
-from baseten.common import api
-from baseten.common.settings import read_config, set_config_value
+from baseten.client_commands import baseten_cli, models_cli, pretrained_cli
+from baseten.common.settings import (read_config, set_config_value,
+                                     set_server_url)
 from baseten.common.util import setup_logger
 
 logger = logging.getLogger(__name__)
@@ -37,11 +37,10 @@ def cli_group():
 @cli_group.command()
 @click.option('--server_url', prompt='BaseTen server URL')
 def configure(server_url):
-    if not _is_valid_url(server_url):
+    if set_server_url(server_url):
+        click.echo('Saved server URL.')
+    else:
         click.echo('That is not a valid URL.')
-        return
-    set_config_value('api', 'url_base', server_url)
-    click.echo('Saved server URL.')
 
 
 @cli_group.command()
@@ -51,20 +50,10 @@ def login(api_key):
     click.echo('Saved API key.')
 
 
-@cli_group.command()
-@ensure_login
-def models():
-    user_models = api.models()
-    click.echo('Your models:\n{}'.format(json.dumps(user_models, indent=4)))
-
-
-def _is_valid_url(url: str) -> bool:
-    try:
-        result = urlparse(url)
-        return all([result.scheme, result.netloc])
-    except ValueError:
-        return False
-
+truss_commands.add_command(baseten_cli.deploy_from_directory)
+cli_group.add_command(truss_commands)
+cli_group.add_command(pretrained_cli.pretrained)
+cli_group.add_command(models_cli.models)
 
 if __name__ == '__main__':
     cli_group()
